@@ -27,7 +27,7 @@ from six import advance_iterator as next
 
 # UFL common.
 from ufl.utils.sorting import sorted_by_key
-from ufl import custom_integral_types
+from ufl.measure import custom_integral_types, point_integral_types
 
 # UFL Classes.
 from ufl.classes import IntValue
@@ -37,7 +37,7 @@ from ufl.classes import Operator
 
 # FFC modules.
 from ffc.log import error, ffc_assert
-from ffc.cpp import format
+from ffc.quadrature.cpp import format
 from ffc.quadrature.quadraturetransformerbase import QuadratureTransformerBase
 from ffc.quadrature.quadratureutils import create_permutations
 
@@ -120,7 +120,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         if permutations:
             for key, val in permutations.items():
                 # Sort key in order to create a unique key.
-                l = sorted(key)
+                l = sorted(key)  # noqa: E741
 
                 # TODO: I think this check can be removed for speed
                 # since we just have a list of objects we should never
@@ -773,7 +773,7 @@ class QuadratureTransformerOpt(QuadratureTransformerBase):
         weight = self._create_symbol(weight, ACCESS)[()]
 
         # Create value.
-        if integral_type in (("vertex",) + custom_integral_types):
+        if integral_type in (point_integral_types + custom_integral_types):
             trans_set = set()
             value = create_product([val, weight])
         else:
