@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 "Quadrature representation class for UFL"
 
-# Copyright (C) 2009-2016 Kristian B. Oelgaard
+# Copyright (C) 2009-2017 Kristian B. Oelgaard
 #
 # This file is part of FFC.
 #
@@ -19,7 +19,7 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg 2009, 2014
-# Modified by Martin Sandve Alnæs 2013-2016
+# Modified by Martin Sandve Alnæs 2013-2017
 
 # Python modules
 import six
@@ -35,9 +35,12 @@ from ffc.log import ffc_assert, info, error
 from ffc.utils import product
 from ffc.fiatinterface import create_element
 
+from ffc.quadrature.cpp import set_float_formatting
+
 from ffc.representationutils import initialize_integral_ir
 from ffc.quadrature.tabulate_basis import tabulate_basis
 from ffc.quadrature.parameters import parse_optimise_parameters
+from ffc.quadrature.deprecation import issue_deprecation_warning
 
 from ffc.quadrature.quadraturetransformer import QuadratureTransformer
 from ffc.quadrature.optimisedquadraturetransformer import QuadratureTransformerOpt
@@ -53,6 +56,10 @@ def compute_integral_ir(itg_data,
 
     info("Computing quadrature representation")
 
+    issue_deprecation_warning()
+
+    set_float_formatting(parameters["precision"])
+
     # Initialise representation
     ir = initialize_integral_ir("quadrature", itg_data, form_data, form_id)
 
@@ -243,7 +250,7 @@ def _extract_element_data(element_map, classnames):
 
             # Get element classname
             element_classname = classnames["finite_element"][ufl_element]
-    
+
             # Store data
             element_data[counter] = {"physical_value_size": value_size,
                                      "num_element_dofs": fiat_element.space_dimension(),
