@@ -28,7 +28,7 @@ from six import advance_iterator as next
 
 # UFL common.
 from ufl.utils.sorting import sorted_by_key
-from ufl import custom_integral_types
+from ufl.measure import custom_integral_types, point_integral_types
 
 # UFL Classes.
 from ufl.classes import IntValue
@@ -38,7 +38,7 @@ from ufl.classes import Operator
 
 # FFC modules.
 from ffc.log import error, ffc_assert
-from ffc.cpp import format
+from ffc.quadrature.cpp import format
 
 # Utility and optimisation functions for quadraturegenerator.
 from ffc.quadrature.quadraturetransformerbase import QuadratureTransformerBase
@@ -158,7 +158,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
         if permutations:
             for key, val in sorted(permutations.items()):
                 # Sort key in order to create a unique key.
-                l = sorted(key)
+                l = sorted(key)  # noqa: E741
 
                 # Loop products, don't multiply by '1' and if we encounter a None the product is zero.
                 # TODO: Need to find a way to remove and J_inv00 terms that might
@@ -841,7 +841,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
             weight += format["component"]("", format["integration points"])
 
         # Update sets of used variables.
-        if integral_type in (("vertex",) + custom_integral_types):
+        if integral_type in (point_integral_types + custom_integral_types):
             trans_set = set()
             value = format["mul"]([val, weight])
         else:
