@@ -24,27 +24,27 @@ transformers to translate UFL expressions."""
 # Modified by Lizao Li, 2015
 # Modified by Anders Logg, 2015
 
-# Python modules.
+# Python modules
+import functools
 from six.moves import zip
 from numpy import shape, array
 
-# UFL Classes.
+# UFL Classes
 from ufl.classes import FixedIndex, Index
 from ufl.utils.stacks import StackDict, Stack
 from ufl.permutation import build_component_numbering
 from ufl import custom_integral_types
 
-# UFL Algorithms.
+# UFL Algorithms
 from ufl.algorithms import Transformer
 
 # FFC modules.
 from ffc.log import ffc_assert, error
-from ffc.fiatinterface import create_element, map_facet_points
-from ffc.mixedelement import MixedElement
-from ffc.cpp import format
+from ffc.fiatinterface import MixedElement, create_element, map_facet_points
+from ffc.quadrature.cpp import format
 
-# FFC tensor modules.
-from ffc.tensor.multiindex import MultiIndex as FFCMultiIndex
+# FFC utils
+from ffc.utils import listcopy
 from ffc.representationutils import transform_component
 
 # Utility and optimisation functions for quadraturegenerator.
@@ -52,6 +52,47 @@ from ffc.quadrature.quadratureutils import create_psi_tables
 from ffc.quadrature.symbolics import BASIS, IP, GEO
 
 
+class FFCMultiIndex:
+    """A MultiIndex represents a list of indices and holds the following
+    data:
+
+        rank    - rank of multiindex
+        dims    - a list of dimensions
+        indices - a list of all possible multiindex values
+
+    """
+
+    def __init__(self, dims):
+        "Create multiindex from given list of ranges"
+
+        def outer_join(a, b):
+            """Let a be a list of lists and b a list. We append each element of b
+            to each list in a and return the resulting list of lists.
+
+            """
+            outer = []
+            for i in range(len(a)):
+                for j in range(len(b)):
+                    outer += [a[i] + [b[j]]]
+            return outer
+
+        def build_indices(dims):
+            "Create a list of all index combinations."
+            if not dims:
+                return [[]]
+            ranges = listcopy(dims)
+            return functools.reduce(outer_join, ranges, [[]])
+
+        self.rank = len(dims)
+        self.dims = [len(dim) for dim in dims]
+        self.indices = build_indices(dims)
+        return
+
+    def __str__(self):
+        "Return informal string representation (pretty-print)."
+        return "rank = %d dims = %s indices = %s" % (self.rank, str(self.dims), str(self.indices))
+
+
 class QuadratureTransformerBase(Transformer):
 
     "Transform UFL representation to quadrature code."
@@ -494,6 +535,10 @@ class QuadratureTransformerBase(Transformer):
 
         if self.vertex is not None:
             error("Spatial coordinates (x) not implemented for point measure (dP)")  # TODO: Implement this, should be just the point.
+        elif self.points is None:
+            gdim, = o.ufl_shape
+            coordinate = "quadrature_points[ip*%d + %d]" % (gdim, c)
+            return self._create_symbol(coordinate, IP)
         else:
             # Generate the appropriate coordinate and update tables.
             coordinate = format["ip coordinates"](self.points, c)
@@ -841,7 +886,7 @@ class QuadratureTransformerBase(Transformer):
         # Try to evaluate entry ("3*6 + 2" --> "20").
         try:
             entry = str(eval(entry))
-        except:
+        except Exception:
             pass
         return loop, entry
 
@@ -1016,7 +1061,7 @@ class QuadratureTransformerBase(Transformer):
         # Try to evaluate basis map ("3 + 2" --> "5").
         try:
             basis_map = str(eval(basis_map))
-        except:
+        except Exception:
             pass
 
         # Create mapping (index, map, loop_range, space_dim).
@@ -1126,7 +1171,7 @@ class QuadratureTransformerBase(Transformer):
         try:
             coefficient_access = str(eval(coefficient_access))
             C_ACCESS = GEO
-        except:
+        except Exception:
             C_ACCESS = IP
         # Format coefficient access
         coefficient = format["coefficient"](str(ufl_function.count()),
@@ -1205,7 +1250,13 @@ class QuadratureTransformerBase(Transformer):
         w, points = self.quad_weights[num_ip]
 
         if self.facet0 is not None:
-            points = map_facet_points(points, self.facet0)
+            # Extract the geometric dimension of the points we want to map
+            dim = len(points[0]) + 1
+
+            dim2cellname = ["interval", "triangle", "tetrahedron"]
+            cellname = dim2cellname[dim-1]
+
+            points = map_facet_points(points, self.facet0, cellname)
             name = f_FEA(num_ip, self.facet0)
         elif self.vertex is not None:
             error("Spatial coordinates (x) not implemented for point measure (dP)")  # TODO: Implement this, should be just the point.
