@@ -13,7 +13,7 @@ function "foo", one should only need to use the data stored
 in the intermediate representation under the key "foo".
 """
 
-# Copyright (C) 2009-2016 Anders Logg, Martin Sandve Alnæs, Marie E. Rognes,
+# Copyright (C) 2009-2017 Anders Logg, Martin Sandve Alnæs, Marie E. Rognes,
 # Kristian B. Oelgaard, and others
 #
 # This file is part of FFC.
@@ -34,21 +34,19 @@ in the intermediate representation under the key "foo".
 # Python modules
 from itertools import chain
 import numpy
+from six.moves import range
 
 # Import UFL
 import ufl
 
 # FFC modules
 from ffc.utils import compute_permutations, product
-from ffc.log import info, error, begin, end, debug_ir, warning
+from ffc.log import info, error, begin, end
 from ffc.fiatinterface import create_element, reference_cell
-from ffc.mixedelement import MixedElement
-from ffc.enrichedelement import EnrichedElement, SpaceOfReals
-from ffc.fiatinterface import HDivTrace
-from ffc.quadratureelement import QuadratureElement
-from ffc.cpp import set_float_formatting
-from ffc.cpp import make_classname, make_integral_classname
+from ffc.fiatinterface import EnrichedElement, HDivTrace, MixedElement, SpaceOfReals, QuadratureElement
+from ffc.classname import make_classname, make_integral_classname
 
+from six import itervalues
 
 # List of supported integral types
 ufc_integral_types = ("cell",
@@ -65,10 +63,10 @@ def pick_representation(representation):
     "Return one of the specialized code generation modules from a representation string."
     if representation == "quadrature":
         from ffc import quadrature as r
-    elif representation == "tensor":
-        from ffc import tensor as r
     elif representation == "uflacs":
         from ffc import uflacs as r
+    elif representation == "tsfc":
+        from ffc import tsfc as r
     else:
         error("Unknown representation: %s" % str(representation))
     return r
@@ -133,7 +131,8 @@ def compute_ir(analysis, prefix, parameters, jit=False):
 
     # Set code generation parameters (this is not actually a 'formatting'
     # parameter, used for table value clamping as well)
-    set_float_formatting(int(parameters["precision"]))
+    # FIXME: Global state?!
+    #    set_float_formatting(parameters["precision"])
 
     # Extract data from analysis
     form_datas, elements, element_numbers, coordinate_elements = analysis
@@ -168,24 +167,24 @@ def compute_ir(analysis, prefix, parameters, jit=False):
 
     # Compute representation of elements
     info("Computing representation of %d elements" % len(elements))
-    ir_elements = [_compute_element_ir(e, element_numbers, classnames, jit)
+    ir_elements = [_compute_element_ir(e, element_numbers, classnames, parameters, jit)
                    for e in elements]
 
     # Compute representation of dofmaps
     info("Computing representation of %d dofmaps" % len(elements))
-    ir_dofmaps = [_compute_dofmap_ir(e, element_numbers, classnames, jit)
+    ir_dofmaps = [_compute_dofmap_ir(e, element_numbers, classnames, parameters, jit)
                   for e in elements]
 
     # Compute representation of coordinate mappings
     info("Computing representation of %d coordinate mappings" % len(coordinate_elements))
-    ir_coordinate_mappings = [_compute_coordinate_mapping_ir(e, element_numbers, classnames, jit)
+    ir_coordinate_mappings = [_compute_coordinate_mapping_ir(e, element_numbers, classnames, parameters, jit)
                               for e in coordinate_elements]
 
     # Compute and flatten representation of integrals
     info("Computing representation of integrals")
     irs = [_compute_integral_ir(fd, form_id, prefix, element_numbers, classnames, parameters, jit)
            for (form_id, fd) in enumerate(form_datas)]
-    ir_integrals = [ir for ir in chain(*irs) if ir is not None]
+    ir_integrals = list(chain(*irs))
 
     # Compute representation of forms
     info("Computing representation of forms")
@@ -197,7 +196,7 @@ def compute_ir(analysis, prefix, parameters, jit=False):
     return ir_elements, ir_dofmaps, ir_coordinate_mappings, ir_integrals, ir_forms
 
 
-def _compute_element_ir(ufl_element, element_numbers, classnames, jit):
+def _compute_element_ir(ufl_element, element_numbers, classnames, parameters, jit):
     "Compute intermediate representation of element."
 
     # Create FIAT element
@@ -224,7 +223,7 @@ def _compute_element_ir(ufl_element, element_numbers, classnames, jit):
     ir["degree"] = ufl_element.degree()
     ir["family"] = ufl_element.family()
 
-    ir["evaluate_basis"] = _evaluate_basis(ufl_element, fiat_element)
+    ir["evaluate_basis"] = _evaluate_basis(ufl_element, fiat_element, parameters["epsilon"])
     ir["evaluate_dof"] = _evaluate_dof(ufl_element, fiat_element)
     ir["interpolate_vertex_values"] = _interpolate_vertex_values(ufl_element,
                                                                  fiat_element)
@@ -239,13 +238,12 @@ def _compute_element_ir(ufl_element, element_numbers, classnames, jit):
     return ir
 
 
-def _compute_dofmap_ir(ufl_element, element_numbers, classnames, jit=False):
+def _compute_dofmap_ir(ufl_element, element_numbers, classnames, parameters, jit=False):
     "Compute intermediate representation of dofmap."
 
     # Create FIAT element
     fiat_element = create_element(ufl_element)
     cell = ufl_element.cell()
-    cellname = cell.cellname()
 
     # Precompute repeatedly used items
     num_dofs_per_entity = _num_dofs_per_entity(fiat_element)
@@ -253,8 +251,7 @@ def _compute_dofmap_ir(ufl_element, element_numbers, classnames, jit=False):
     facet_dofs = _tabulate_facet_dofs(fiat_element, cell)
     entity_closure_dofs, num_dofs_per_entity_closure = \
         _tabulate_entity_closure_dofs(fiat_element, cell)
-    
-    
+
     # Store id
     ir = {"id": element_numbers[ufl_element]}
     ir["classname"] = classnames["dofmap"][ufl_element]
@@ -268,6 +265,8 @@ def _compute_dofmap_ir(ufl_element, element_numbers, classnames, jit=False):
     ir["topological_dimension"] = cell.topological_dimension()
     ir["geometric_dimension"] = cell.geometric_dimension()
     ir["global_dimension"] = _global_dimension(fiat_element)
+    ir["num_global_support_dofs"] = _num_global_support_dofs(fiat_element)
+    ir["num_element_support_dofs"] = fiat_element.space_dimension() - ir["num_global_support_dofs"]
     ir["num_element_dofs"] = fiat_element.space_dimension()
     ir["num_facet_dofs"] = len(facet_dofs[0])
     ir["num_entity_dofs"] = num_dofs_per_entity
@@ -337,7 +336,7 @@ def _tabulate_coordinate_mapping_basis(ufl_element):
 
 
 def _compute_coordinate_mapping_ir(ufl_coordinate_element, element_numbers,
-                                   classnames, jit=False):
+                                   classnames, parameters, jit=False):
     "Compute intermediate representation of coordinate mapping."
 
     cell = ufl_coordinate_element.cell()
@@ -390,6 +389,19 @@ def _compute_coordinate_mapping_ir(ufl_coordinate_element, element_numbers,
     return ir
 
 
+def _num_global_support_dofs(fiat_element):
+    "Compute number of global support dofs."
+    if not isinstance(fiat_element, MixedElement):
+        if isinstance(fiat_element, SpaceOfReals):
+            return 1
+        return 0
+    num_reals = 0
+    for e in fiat_element.elements():
+        if isinstance(e, SpaceOfReals):
+            num_reals += 1
+    return num_reals
+
+
 def _global_dimension(fiat_element):
     "Compute intermediate representation for global_dimension."
 
@@ -399,14 +411,15 @@ def _global_dimension(fiat_element):
         return (_num_dofs_per_entity(fiat_element), 0)
 
     elements = []
-    reals = []
     num_reals = 0
-    for (i, e) in enumerate(fiat_element.elements()):
+    for e in fiat_element.elements():
         if not isinstance(e, SpaceOfReals):
             elements += [e]
         else:
             num_reals += 1
-    fiat_element = MixedElement(elements)
+    fiat_cell, = set(e.get_reference_element()
+                     for e in fiat_element.elements())
+    fiat_element = MixedElement(elements, ref_el=fiat_cell)
     return (_num_dofs_per_entity(fiat_element), num_reals)
 
 
@@ -449,6 +462,9 @@ def _compute_integral_ir(form_data, form_id, prefix, element_numbers, classnames
                                    classnames,
                                    parameters)
 
+        # Remember jit status
+        ir["jit"] = jit
+
         # Build classname
         ir["classname"] = make_integral_classname(prefix, itg_data.integral_type,
                                                   form_id, itg_data.subdomain_id)
@@ -604,9 +620,6 @@ def _generate_physical_offsets(ufl_element, offset=0):
 def _generate_offsets(ufl_element, reference_offset=0, physical_offset=0):
     """Generate offsets: i.e value offset for each basis function
     relative to a physical element representation."""
-    cell = ufl_element.cell()
-    gdim = cell.geometric_dimension()
-    tdim = cell.topological_dimension()
 
     if isinstance(ufl_element, ufl.MixedElement):
         offsets = []
@@ -638,13 +651,18 @@ def _generate_offsets(ufl_element, reference_offset=0, physical_offset=0):
 def _evaluate_dof(ufl_element, fiat_element):
     "Compute intermediate representation of evaluate_dof."
     cell = ufl_element.cell()
+    if fiat_element.is_nodal():
+        dofs = [L.pt_dict for L in fiat_element.dual_basis()]
+    else:
+        dofs = [None] * fiat_element.space_dimension()
     return {"mappings": fiat_element.mapping(),
             "reference_value_size": ufl_element.reference_value_size(),
             "physical_value_size": ufl_element.value_size(),
             "geometric_dimension": cell.geometric_dimension(),
             "topological_dimension": cell.topological_dimension(),
-            "dofs": [L.pt_dict if L else None for L in fiat_element.dual_basis()],
-            "physical_offsets": _generate_physical_offsets(ufl_element)}
+            "dofs": dofs,
+            "physical_offsets": _generate_physical_offsets(ufl_element),
+            "cell_shape": cell.cellname()}
 
 
 def _extract_elements(fiat_element):
@@ -657,7 +675,7 @@ def _extract_elements(fiat_element):
     return new_elements
 
 
-def _evaluate_basis(ufl_element, fiat_element):
+def _evaluate_basis(ufl_element, fiat_element, epsilon):
     "Compute intermediate representation for evaluate_basis."
     cell = ufl_element.cell()
     cellname = cell.cellname()
@@ -682,6 +700,13 @@ def _evaluate_basis(ufl_element, fiat_element):
         if isinstance(e, HDivTrace):
             return "Function not supported for Trace elements"
 
+    # Skip this function for TensorProductElement if get_coeffs is not implemented
+    for e in elements:
+        try:
+            e.get_coeffs()
+        except NotImplementedError:
+            return "Function is not supported/implemented."
+
     # Initialise data with 'global' values.
     data = {"reference_value_size": ufl_element.reference_value_size(),
             "physical_value_size": ufl_element.value_size(),
@@ -702,6 +727,10 @@ def _evaluate_basis(ufl_element, fiat_element):
         num_expansion_members = e.get_num_members(e.degree())
         dmats = e.dmats()
 
+        # Clamp dmats zeros
+        dmats = numpy.asarray(dmats)
+        dmats[numpy.where(numpy.isclose(dmats, 0.0, rtol=epsilon, atol=epsilon))] = 0.0
+
         # Extracted parts of dd below that are common for the element
         # here.  These dict entries are added to each dof_data dict
         # for each dof, because that's what the code generation
@@ -733,6 +762,10 @@ def _evaluate_basis(ufl_element, fiat_element):
             else:
                 error("Unknown situation with num_components > 1")
 
+            # Clamp coefficient zeros
+            coefficients = numpy.asarray(coefficients)
+            coefficients[numpy.where(numpy.isclose(coefficients, 0.0, rtol=epsilon, atol=epsilon))] = 0.0
+
             dof_data = {
                 "coeffs": coefficients,
                 "mapping": mappings[dof],
@@ -753,8 +786,12 @@ def _evaluate_basis(ufl_element, fiat_element):
 
 def _tabulate_dof_coordinates(ufl_element, element):
     "Compute intermediate representation of tabulate_dof_coordinates."
+    if uses_integral_moments(element):
+        return {}
 
-    if uses_integral_moments(element) or not element.dual_basis()[0]:
+    # Bail out if any dual basis member is missing (element is not nodal),
+    # this is strictly not necessary but simpler
+    if any(L is None for L in element.dual_basis()):
         return {}
 
     cell = ufl_element.cell()
@@ -763,6 +800,7 @@ def _tabulate_dof_coordinates(ufl_element, element):
     data["tdim"] = cell.topological_dimension()
     data["gdim"] = cell.geometric_dimension()
     data["points"] = [sorted(L.pt_dict.keys())[0] for L in element.dual_basis()]
+    data["cell_shape"] = cell.cellname()
     return data
 
 
@@ -799,67 +837,37 @@ def _tabulate_dofs(element, cell):
 def _tabulate_facet_dofs(element, cell):
     "Compute intermediate representation of tabulate_facet_dofs."
 
-    # Compute incidences
-    incidence = __compute_incidence(cell.topological_dimension())
-
     # Get topological dimension
-    D = max([pair[0][0] for pair in incidence])
+    D = cell.topological_dimension()
 
     # Get the number of facets
     num_facets = cell.num_facets()
 
-    # Find out which entities are incident to each facet
-    incident = num_facets * [None]
-    for facet in range(num_facets):
-        incident[facet] = [pair[1] for pair in incidence
-                           if incidence[pair] is True and pair[0] == (D - 1, facet)]
-
     # Make list of dofs
-    facet_dofs = []
-    entity_dofs = element.entity_dofs()
+    facet_dofs = list(itervalues(element.entity_closure_dofs()[D-1]))
+
+    assert num_facets == len(facet_dofs)
+
+    facet_dofs = [facet_dofs[facet] for facet in range(num_facets)]
 
-    for facet in range(num_facets):
-        facet_dofs += [[]]
-        for dim in entity_dofs:
-            for entity in entity_dofs[dim]:
-                if (dim, entity) in incident[facet]:
-                    facet_dofs[facet] += entity_dofs[dim][entity]
-        facet_dofs[facet].sort()
     return facet_dofs
 
 
 def _tabulate_entity_closure_dofs(element, cell):
     "Compute intermediate representation of tabulate_entity_closure_dofs."
 
-    # Compute incidences
-    incidence = __compute_incidence(cell.topological_dimension())
-
     # Get topological dimension
-    D = max([pair[0][0] for pair in incidence])
+    D = cell.topological_dimension()
 
-    entity_dofs = element.entity_dofs()
+    # Get entity closure dofs from FIAT element
+    fiat_entity_closure_dofs = element.entity_closure_dofs()
 
     entity_closure_dofs = {}
-    for d0 in range(D + 1):
-        # Find out which entities are incident to each entity of dim d0
-        incident = {}
-        for e0 in entity_dofs[d0]:
-            incident[(d0, e0)] = [pair[1] for pair in incidence
-                                  if incidence[pair] is True and pair[0] == (d0, e0)]
+    for d0 in sorted(fiat_entity_closure_dofs.keys()):
+        for e0 in sorted(fiat_entity_closure_dofs[d0].keys()):
+            entity_closure_dofs[(d0, e0)] = fiat_entity_closure_dofs[d0][e0]
 
-        # Make list of dofs
-        for e0 in entity_dofs[d0]:
-            dofs = []
-            for d1 in entity_dofs:
-                for e1 in entity_dofs[d1]:
-                    if (d1, e1) in incident[(d0, e0)]:
-                        dofs += entity_dofs[d1][e1]
-            entity_closure_dofs[(d0, e0)] = sorted(dofs)
-
-    num_entity_closure_dofs = [max(len(dofs)
-                               for (d, e), dofs in entity_closure_dofs.items()
-                               if d == dim)
-                           for dim in range(D + 1)]
+    num_entity_closure_dofs = [len(fiat_entity_closure_dofs[d0][0]) for d0 in sorted(fiat_entity_closure_dofs.keys())]
 
     return entity_closure_dofs, num_entity_closure_dofs
 
@@ -975,58 +983,7 @@ def _num_dofs_per_entity(fiat_element):
     Example: Lagrange of degree 3 on triangle: [1, 2, 1]
     """
     entity_dofs = fiat_element.entity_dofs()
-    return [len(entity_dofs[e][0]) for e in range(len(entity_dofs.keys()))]
-
-# These two are copied from old ffc
-
-
-def __compute_incidence(D):
-    "Compute which entities are incident with which"
-
-    # Compute the incident vertices for each entity
-    sub_simplices = []
-    for dim in range(D + 1):
-        sub_simplices += [__compute_sub_simplices(D, dim)]
-
-    # Check which entities are incident, d0 --> d1 for d0 >= d1
-    incidence = {}
-    for d0 in range(0, D + 1):
-        for i0 in range(len(sub_simplices[d0])):
-            for d1 in range(d0 + 1):
-                for i1 in range(len(sub_simplices[d1])):
-                    incidence[((d0, i0), (d1, i1))] = all(v in sub_simplices[d0][i0]
-                                                          for v in sub_simplices[d1][i1])
-    return incidence
-
-
-def __compute_sub_simplices(D, d):
-    """Compute vertices for all sub simplices of dimension d (code
-    taken from Exterior)."""
-
-    # Number of vertices
-    num_vertices = D + 1
-
-    # Special cases: d = 0 and d = D
-    if d == 0:
-        return [[i] for i in range(num_vertices)]
-    elif d == D:
-        return [list(range(num_vertices))]
-
-    # Compute all permutations of num_vertices - (d + 1)
-    permutations = compute_permutations(num_vertices - d - 1, num_vertices)
-
-    # Iterate over sub simplices
-    sub_simplices = []
-    for i in range(len(permutations)):
-
-        # Pick tuple i among permutations (non-incident vertices)
-        remove = permutations[i]
-
-        # Remove vertices, keeping d + 1 vertices
-        vertices = [v for v in range(num_vertices) if v not in remove]
-        sub_simplices += [vertices]
-
-    return sub_simplices
+    return [len(entity_dofs[e][0]) for e in sorted(entity_dofs.keys())]
 
 
 def uses_integral_moments(fiat_element):
