@@ -28,7 +28,7 @@ option --bench.
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
-# Modified by Martin Sandve Alnæs, 2013-2016
+# Modified by Martin Sandve Alnæs, 2013-2017
 # Modified by Johannes Ring, 2013
 # Modified by Kristian B. Oelgaard, 2013
 # Modified by Garth N. Wells, 2014
@@ -36,6 +36,7 @@ option --bench.
 # FIXME: Need to add many more test cases. Quite a few DOLFIN forms
 # failed after the FFC tests passed.
 
+from collections import OrderedDict, defaultdict
 import os
 import sys
 import shutil
@@ -44,20 +45,19 @@ import sysconfig
 import subprocess
 import time
 import logging
+import traceback
 from numpy import array, shape, abs, max, isnan
 import ffc
 from ffc.log import begin, end, info, info_red, info_green, info_blue
-from ffc.log import ffc_logger, ERROR, WARNING
+from ffc.log import ffc_logger, ERROR
 from ufl.log import ufl_logger
 from ufl.utils.py23 import as_native_str
 from ffc import get_ufc_cxx_flags
 from ffc.backends.ufc import get_include_path as get_ufc_include
 from ufctest import generate_test_code
 
-# Parameters TODO: Can make this a cmdline argument, and start
-# crashing programs in debugger automatically?
-debug = False
-output_tolerance = 1.e-6
+# Parameters
+output_tolerance = 1e-5
 demo_directory = "../../../../demo"
 bench_directory = "../../../../bench"
 
@@ -84,6 +84,7 @@ class LTFilter(object):
     def filter(self, record):
         return record.levelno < self.__level
 
+
 # Filter out error messages from std output
 splitlevel = ERROR
 ffc_logger.get_handler().addFilter(LTFilter(splitlevel))
@@ -105,22 +106,39 @@ ext_quad = [
     "-r quadrature -O -fprecompute_basis_const -feliminate_zeros",
 ]
 
-# Extended uflacs tests (to be extended with optimisation parameters
-# later)
+# Extended uflacs tests
+# (to be extended with optimisation parameters later)
 ext_uflacs = [
-    "-r uflacs",
+    "-r uflacs -O -fvectorize -fpadlen=4 -falignas=32",
+    "-r uflacs -O -fno-enable_sum_factorization",
+    "-r uflacs -O -fno-enable_preintegration",
+    "-r uflacs -O -fenable_premultiplication",
 ]
 
+known_quad_failures = set([
+    "PoissonQuad.ufl",
+])
+
 known_uflacs_failures = set([
     "CustomIntegral.ufl",
     "CustomMixedIntegral.ufl",
     "CustomVectorIntegral.ufl",
+    "MetaData.ufl",
 ])
 
+known_tsfc_failures = set([
+    # Expected not to work
+    "CustomIntegral.ufl",
+    "CustomMixedIntegral.ufl",
+    "CustomVectorIntegral.ufl",
+    "MetaData.ufl",
+])
+
+
 _command_timings = []
 
 
-def run_command(command):
+def run_command(command, verbose):
     "Run command and collect errors in log file."
     global _command_timings
 
@@ -129,15 +147,15 @@ def run_command(command):
         output = as_native_str(subprocess.check_output(command, shell=True))
         t2 = time.time()
         _command_timings.append((command, t2 - t1))
-        verbose = False  # FIXME: Set from --verbose
         if verbose:
             print(output)
         return True
     except subprocess.CalledProcessError as e:
         t2 = time.time()
         _command_timings.append((command, t2 - t1))
-        log_error(e.output)
-        print(e.output)
+        if e.output:
+            log_error(e.output)
+            print(e.output)
         return False
 
 
@@ -190,7 +208,7 @@ def generate_test_cases(bench, only_forms, skip_forms):
     end()
 
 
-def generate_code(args, only_forms, skip_forms):
+def generate_code(args, only_forms, skip_forms, debug):
     "Generate code for all test cases."
     global _command_timings
 
@@ -207,11 +225,13 @@ def generate_code(args, only_forms, skip_forms):
     # some sort of tag like '#ffc: <flags>'.
     special = {"AdaptivePoisson.ufl": "-e", }
 
+    failures = []
+
     # Iterate over all files
     for f in form_files:
         options = [special.get(f, "")]
         options.extend(args)
-        options.extend(["-f", "precision=8", "-fconvert_exceptions_to_warnings"])
+        options.extend(["-f", "precision=8", "-f", "epsilon=1e-7",  "-fconvert_exceptions_to_warnings"])
         options.append(f)
         options = list(filter(None, options))
 
@@ -222,9 +242,12 @@ def generate_code(args, only_forms, skip_forms):
         try:
             ok = ffc.main(options)
         except Exception as e:
-            log_error(e)
+            if debug:
+                raise e
+            msg = traceback.format_exc()
+            log_error(cmd)
+            log_error(msg)
             ok = 1
-            raise
         finally:
             t2 = time.time()
             _command_timings.append((cmd, t2 - t1))
@@ -234,8 +257,10 @@ def generate_code(args, only_forms, skip_forms):
             info_green("%s OK" % f)
         else:
             info_red("%s failed" % f)
+            failures.append(f)
 
     end()
+    return failures
 
 
 def validate_code(reference_dir):
@@ -247,6 +272,8 @@ def validate_code(reference_dir):
     begin("Validating generated code (%d header files found)"
           % len(header_files))
 
+    failures = []
+
     # Iterate over all files
     for f in header_files:
 
@@ -274,8 +301,10 @@ def validate_code(reference_dir):
                  % os.path.join(*reference_file.split(os.path.sep)[-3:]))
             log_error("\n" + s + "\n" + len(s) * "-")
             log_error(diff)
+            failures.append(f)
 
     end()
+    return failures
 
 
 def find_boost_cflags():
@@ -325,7 +354,7 @@ Forms using bessel functions will fail to build.
     return boost_cflags, boost_linkflags
 
 
-def build_programs(bench, permissive):
+def build_programs(bench, permissive, debug, verbose):
     "Build test programs for all test cases."
 
     # Get a list of all files
@@ -335,47 +364,75 @@ def build_programs(bench, permissive):
 
     # Get UFC flags
     ufc_cflags = "-I" + get_ufc_include() + " " + " ".join(get_ufc_cxx_flags())
+
+    # Get boost flags
     boost_cflags, boost_linkflags = find_boost_cflags()
-    ufc_cflags += boost_cflags
-    linker_options = boost_linkflags
+
+    # Get compiler
+    compiler = os.getenv("CXX", "g++")
 
     # Set compiler options
-    compiler_options = "%s -Wall " % ufc_cflags
+    compiler_options = " -Wall"
     if not permissive:
         compiler_options += " -Werror -pedantic"
+
+    # Always need ufc
+    compiler_options += " " + ufc_cflags
+
     if bench:
         info("Benchmarking activated")
-        # Takes too long to build with -O2
-        # compiler_options += " -O2"
-        compiler_options += " -O3"
-        # compiler_options += " -O3 -fno-math-errno -march=native"
+        compiler_options += " -O3 -march=native"
+        # Workaround for gcc bug: gcc is too eager to report array-bounds warning with -O3
+        compiler_options += " -Wno-array-bounds"
+
     if debug:
         info("Debugging activated")
         compiler_options += " -g -O0"
+
     info("Compiler options: %s" % compiler_options)
 
+    failures = []
+
     # Iterate over all files
     for f in header_files:
+        prefix = f.split(".h")[0]
+
+        # Options for all files
+        cpp_flags = compiler_options
+        ld_flags = ""
+
+        # Only add boost flags if necessary
+        needs_boost = prefix == "MathFunctions"
+        if needs_boost:
+            info("Additional compiler options for %s: %s" % (prefix, boost_cflags))
+            info("Additional linker options for %s: %s" % (prefix, boost_linkflags))
+            cpp_flags += " " + boost_cflags
+            ld_flags += " " + boost_linkflags
 
         # Generate test code
         filename = generate_test_code(f)
 
         # Compile test code
-        prefix = f.split(".h")[0]
-        command = "g++ %s -o %s.bin %s.cpp %s" % \
-                  (compiler_options, prefix, prefix, linker_options)
-        ok = run_command(command)
+        command = "%s %s -o %s.bin %s.cpp %s" % \
+                  (compiler, cpp_flags, prefix, prefix, ld_flags)
+        ok = run_command(command, verbose)
+
+        # Store compile command for easy reproduction
+        with open("%s.build" % (prefix,), "w") as f:
+            f.write(command + "\n")
 
         # Check status
         if ok:
             info_green("%s OK" % prefix)
         else:
             info_red("%s failed" % prefix)
+            failures.append(prefix)
 
     end()
+    return failures
 
 
-def run_programs(bench):
+def run_programs(bench, debug, verbose):
     "Run generated programs."
 
     # This matches argument parsing in the generated main files
@@ -386,20 +443,22 @@ def run_programs(bench):
 
     begin("Running generated programs (%d programs found)" % len(test_programs))
 
+    failures = []
+
     # Iterate over all files
     for f in test_programs:
-
         # Compile test code
         prefix = f.split(".bin")[0]
-        ok = run_command(".%s%s.bin %s" % (os.path.sep, prefix, bench))
+        ok = run_command(".%s%s.bin %s" % (os.path.sep, prefix, bench), verbose)
 
         # Check status
         if ok:
             info_green("%s OK" % f)
         else:
             info_red("%s failed" % f)
-
+            failures.append(f)
     end()
+    return failures
 
 
 def validate_programs(reference_dir):
@@ -411,6 +470,8 @@ def validate_programs(reference_dir):
     begin("Validating generated programs (%d .json program output files found)"
           % len(output_files))
 
+    failures = []
+
     # Iterate over all files
     for fj in output_files:
 
@@ -455,40 +516,51 @@ def validate_programs(reference_dir):
             log_error("Json output differs for %s, diff follows (generated first, reference second)"
                       % os.path.join(*reference_json_file.split(os.path.sep)[-3:]))
             print_recdiff(json_diff, printer=log_error)
+            failures.append(fj)
 
     end()
+    return failures
 
 
 def main(args):
     "Run all regression tests."
 
     # Check command-line arguments TODO: Use argparse
-    use_auto = "--skip-auto" not in args
+    only_auto  = "--only-auto" in args
+    use_auto   = "--skip-auto" not in args
     use_uflacs = "--skip-uflacs" not in args
-    use_quad = "--skip-quad" not in args
-    use_ext_quad = "--ext-quad" in args
+    use_quad   = "--skip-quad" not in args
+    use_tsfc   = "--use-tsfc" in args
+    use_ext_quad   = "--ext-quad" in args
+    use_ext_uflacs = "--ext-uflacs" in args
 
     skip_download = "--skip-download" in args
     skip_run = "--skip-run" in args
     skip_code_diff = "--skip-code-diff" in args
     skip_validate = "--skip-validate" in args
     bench = "--bench" in args
+    debug = "--debug" in args
+    verbose = ("--verbose" in args) or debug  # debug implies verbose
 
-    permissive = "--permissive" in args
+    permissive = "--permissive" in args or bench
     tolerant = "--tolerant" in args
     print_timing = "--print-timing" in args
     show_help = "--help" in args
 
     flags = (
+        "--only-auto",
         "--skip-auto",
         "--skip-uflacs",
         "--skip-quad",
+        "--use-tsfc",
         "--ext-quad",
         "--skip-download",
         "--skip-run",
         "--skip-code-diff",
         "--skip-validate",
         "--bench",
+        "--debug",
+        "--verbose",
         "--permissive",
         "--tolerant",
         "--print-timing",
@@ -496,6 +568,10 @@ def main(args):
     )
     args = [arg for arg in args if arg not in flags]
 
+    # Hack: add back --verbose for ffc.main to see
+    if verbose:
+        args = args + ["--verbose"]
+
     if show_help:
         info("Valid arguments:\n" + "\n".join(flags))
         return 0
@@ -505,6 +581,8 @@ def main(args):
     if bench:
         skip_code_diff = True
         skip_validate = True
+    if use_ext_quad or use_ext_uflacs:
+        skip_code_diff = True
 
     # Extract .ufl names from args
     only_forms = set([arg for arg in args if arg.endswith(".ufl")])
@@ -534,19 +612,35 @@ def main(args):
 
     # Adjust which test cases (combinations of compile arguments) to run here
     test_cases = []
-    if use_auto:
+    if only_auto:
         test_cases += ["-r auto"]
-    if use_uflacs:
-        test_cases += ["-r uflacs"]
-    if use_quad:
-        test_cases += ["-r quadrature", "-r quadrature -O"]
-    if use_ext_quad:
-        test_cases += ext_quad
+    else:
+        if use_auto:
+            test_cases += ["-r auto"]
+        if use_uflacs:
+            test_cases += ["-r uflacs -O0", "-r uflacs -O"]
+        if use_quad:
+            test_cases += ["-r quadrature -O0", "-r quadrature -O"]
+            import warnings
+            from ffc.quadrature.deprecation import QuadratureRepresentationDeprecationWarning
+            warnings.simplefilter("once", QuadratureRepresentationDeprecationWarning)
+        if use_tsfc:
+            test_cases += ["-r tsfc -O0", "-r tsfc -O"]
+            # Silence good-performance messages by COFFEE
+            import coffee
+            coffee.set_log_level(coffee.logger.PERF_WARN)
+        if use_ext_quad:
+            test_cases += ext_quad
+        if use_ext_uflacs:
+            test_cases += ext_uflacs
 
     test_case_timings = {}
 
+    fails = OrderedDict()
+
     for argument in test_cases:
         test_case_timings[argument] = time.time()
+        fails[argument] = OrderedDict()
 
         begin("Running regression tests with %s" % argument)
 
@@ -555,10 +649,16 @@ def main(args):
         clean_output(sub_directory)
         os.chdir(sub_directory)
 
-        # Workarounds for partial feature completeness in uflacs
-        if "uflacs" in argument and not only_forms:
+        # Workarounds for feature lack in representation
+        if "quadrature" in argument and not only_forms:
+            skip_forms = known_quad_failures
+            info_blue("Skipping forms known to fail with quadrature:\n" + "\n".join(sorted(skip_forms)))
+        elif "uflacs" in argument and not only_forms:
             skip_forms = known_uflacs_failures
             info_blue("Skipping forms known to fail with uflacs:\n" + "\n".join(sorted(skip_forms)))
+        elif "tsfc" in argument and not only_forms:
+            skip_forms = known_tsfc_failures
+            info_blue("Skipping forms known to fail with tsfc:\n" + "\n".join(sorted(skip_forms)))
         else:
             skip_forms = set()
 
@@ -566,7 +666,9 @@ def main(args):
         generate_test_cases(bench, only_forms, skip_forms)
 
         # Generate code
-        generate_code(args + argument.split(), only_forms, skip_forms)
+        failures = generate_code(args + argument.split(), only_forms, skip_forms, debug)
+        if failures:
+            fails[argument]["generate_code"] = failures
 
         # Location of reference directories
         reference_directory = os.path.abspath("../../ffc-reference-data/")
@@ -579,23 +681,33 @@ def main(args):
 
         # Validate code by comparing to code generated with this set
         # of compiler parameters
-        if skip_code_diff or (argument in ext_quad):
+        if skip_code_diff:
             info_blue("Skipping code diff validation")
         else:
-            validate_code(code_reference_dir)
+            failures = validate_code(code_reference_dir)
+            if failures:
+                fails[argument]["validate_code"] = failures
 
         # Build and run programs and validate output to common
         # reference
         if skip_run:
             info_blue("Skipping program execution")
         else:
-            build_programs(bench, permissive)
-            run_programs(bench)
+            failures = build_programs(bench, permissive, debug, verbose)
+            if failures:
+                fails[argument]["build_programs"] = failures
+
+            failures = run_programs(bench, debug, verbose)
+            if failures:
+                fails[argument]["run_programs"] = failures
+
             # Validate output to common reference results
             if skip_validate:
                 info_blue("Skipping program output validation")
             else:
-                validate_programs(output_reference_dir)
+                failures = validate_programs(output_reference_dir)
+                if failures:
+                    fails[argument]["validate_programs"] = failures
 
         # Go back up
         os.chdir(os.path.pardir)
@@ -616,11 +728,42 @@ def main(args):
     for argument in test_cases:
         info_blue("Total time for %s: %.1f s" % (argument, test_case_timings[argument]))
 
-    if not os.path.isfile(logfile) or os.stat(logfile).st_size == 0:
+    num_failures = sum(len(failures_phase)
+                       for failures_args in fails.values()
+                       for failures_phase in failures_args.values())
+
+    if num_failures == 0:
         info_green("Regression tests OK")
         return 0
     else:
         info_red("Regression tests failed")
+        info_red("")
+        info_red("Long summary:")
+        for argument in test_cases:
+            if not fails[argument]:
+                info_green("  No failures with args '%s'" % argument)
+            else:
+                info_red("  Failures with args '%s':" % argument)
+                for phase, failures in fails[argument].items():
+                    info_red("    %d failures in %s:" % (len(failures), phase))
+                    for f in failures:
+                        info_red("      %s" % (f,))
+        info_red("")
+        info_red("Short summary:")
+        phase_fails = defaultdict(int)
+        for argument in test_cases:
+            if not fails[argument]:
+                info_green("  No failures with args '%s'" % argument)
+            else:
+                info_red("  Number of failures with args '%s':" % argument)
+                for phase, failures in fails[argument].items():
+                    info_red("    %d failures in %s." % (len(failures), phase))
+                    phase_fails[phase] += len(failures)
+        info_red("")
+        info_red("Total failures for all args:")
+        for phase, count in phase_fails.items():
+            info_red("    %s: %d failed" % (phase, count))
+        info_red("")
         info_red("Error messages stored in %s" % logfile)
         return 1
 
