@@ -1,14 +1,15 @@
-#!/usr/bin/env python
+# -*- coding: utf-8 -*-
 """
-Tests of CRS data structure.
+Tests of CRSArray data structure.
 """
 
 from six.moves import xrange
-from uflacs.datastructures.crs import CRS
+from ffc.uflacs.analysis.crsarray import CRSArray
+
 
 def test_crs_can_have_zero_element_rows():
     rcap, ecap = 3, 1
-    A = CRS(rcap, ecap, int)
+    A = CRSArray(rcap, ecap, int)
     for i in range(rcap):
         row = []
         A.push_row(row)
@@ -17,9 +18,10 @@ def test_crs_can_have_zero_element_rows():
         row = []
         assert list(A[i]) == row
 
+
 def test_crs_can_have_one_element_rows():
     rcap, ecap = 3, 3
-    A = CRS(rcap, ecap, int)
+    A = CRSArray(rcap, ecap, int)
     for i in range(rcap):
         row = [i]
         A.push_row(row)
@@ -28,15 +30,16 @@ def test_crs_can_have_one_element_rows():
         row = [i]
         assert list(A[i]) == row
 
+
 def test_crs_can_have_n_element_rows():
     rcap, ecap = 5, 25
-    A = CRS(rcap, ecap, int)
+    A = CRSArray(rcap, ecap, int)
     k = 0
     for i in range(rcap):
-        row = [i+2, i+1] + [i]*i
+        row = [i + 2, i + 1] + [i] * i
         k += len(row)
         A.push_row(row)
     assert A.num_elements == k
     for i in range(rcap):
-        row = [i+2, i+1] + [i]*i
+        row = [i + 2, i + 1] + [i] * i
         assert list(A[i]) == row
