@@ -13,7 +13,7 @@ from ufl.permutation import compute_indices
 from ffc.uflacs.analysis.graph import build_graph
 from ffc.uflacs.analysis.graph_rebuild import rebuild_expression_from_graph
 # from ffc.uflacs.analysis.graph_rebuild import rebuild_scalar_e2i
-# from ffc.uflacs.analysis.graph_dependencies import (compute_dependencies,
+# from ffc.uflacs.analysis.dependencies import (compute_dependencies,
 #                                                mark_active,
 #                                                mark_image)
 # from ffc.uflacs.analysis.graph_ssa import (mark_partitions,
@@ -150,7 +150,7 @@ def test_rebuild_expression_from_graph_on_products_with_indices():
     u = Coefficient(U)
     v = Coefficient(V)
     w = Coefficient(W)
-    i, j, k, l = indices(4)
+    i, j = indices(2)
 
     # Test fixed index
     fixed = [u * v[0], v[1] * v[0], w[0, 1] * w[0, 0]]
